--EyeTV Exporter Version 1.3
--
--EyeTV Exporter is little Applescript to export all your EyeTV Recording with one click.
--The exported recordings will be found in your Movie Folder. You could choose the Exportformat at start.
--The exported filenames will contain title, startdate, starttime and exportformat.
--
--Script made by Jan Kampling
--http://www.macmediacenter.de
--Licenesed under GNU General Public License, Free Software Foundation http://creativecommons.org/licenses/GPL/2.0/
--
--
-- Changelog: 
-- Version 1.3:
--	+ added new exportformats from EyeTV 1.8.4 (iPod and iPodMP4)
-- Version 1.2.1:
--	+Bugfix Growlfunction
-- Version 1.2:
--	+ checks how many jobs EyeTv is running (exports and recordings) and never start more than 2 jobs at one time. So the exports are going faster and EyeTv have enough CPU time to record and you have enough CPU to work while exoprting
--	+ should now run with and without Growl installed
--	+ added new Exportformats from EyeTV 1.8.3 - PSPH264, AAC, Applelossless
--
-- Version 1.1:
--	+ use converttext routine to replace problematic characters in Recordingtitles and 
-- 	+ changed the way the date is converted to a string (so no more "/", ":"; "*" in title and time, important for SMB sharing)
--	+ Date and Time is now in the language Mac OS is running
--	+ removed unneeded Application specifical Exportformats (like Toast, VCD, SVCD, etc)
--	+ Dialog if there are no recordings to export
--	+ you could now choose if you want to use the standard folder or the folder you choose
--
--
-- You could change the follwing variables
-- Foldername of EyeTV Exporter
property myFoldername : "EyeTV Export"
--
-- Sub-Routines
--
-- load external subroutines
--
--
--set myCoolLib to load script file "Users:jan:Documents:EyeTV Exporter:eyetvexporter-1.1:SourceCode:baseLib_2.2.1_008"
--
--
-- Sub-Routine for converting illegal characters in filename strings
property searchList : {":", "/", "*"}
property replaceList : {"-", "-", "-"}

to switchText of t from s to r
	set text item delimiters to s
	set t to t's text items
	set text item delimiters to r
	tell t to set t to beginning & ({""} & rest)
	t
end switchText

to convertText(t)
	set d to text item delimiters
	repeat with n from 1 to count searchList
		set t to switchText of t from my searchList's item n to my replaceList's item n
	end repeat
	set text item delimiters to d
	t
end convertText
--
on add_leading_zeros(this_number, max_leading_zeros)
	set the threshold_number to (10 ^ max_leading_zeros) as integer
	if this_number is less than the threshold_number then
		set the leading_zeros to ""
		set the digit_count to the length of ((this_number div 1) as string)
		set the character_count to (max_leading_zeros + 1) - digit_count
		repeat character_count times
			set the leading_zeros to (the leading_zeros & "0") as string
		end repeat
		return (leading_zeros & (this_number as text)) as string
	else
		return this_number as text
	end if
end add_leading_zeros
--
--
-- timeString -- by Richard Morton 2000
-- Return a file name friendly time string from a date object
on timeString(dateObj)
	set timeStr to time string of dateObj
	tell AppleScript -- Just saves typing
		set oD to text item delimiters
		set text item delimiters to ":" -- assumes colon time separator
		set timeStrItems to text items of timeStr
		set text item delimiters to "-" -- change to something else if req.
		set newTimeStr to timeStrItems as string
		set text item delimiters to oD
		return newTimeStr
	end tell
end timeString
--
on counteyetvproc()
	tell application "EyeTV"
		try
			launch -- apps can't receive the run event except at startup, from the Finder, so this causes an error 
		on error
			display dialog "Could not start EyeTV, perhaps it is not installed" buttons {"OK"} default button 1
		end try
		--
		set myCounter to 0
		repeat with myRecording in recordings
			if busy of myRecording is true then
				set myCounter to myCounter + 1
			end if
		end repeat
	end tell
	return myCounter
end counteyetvproc
--
on finderstuff(myTargetPath, myMovieFolder, myFolder)
	tell application "Finder"
		if folder myTargetPath exists then
			-- do nothing
		else
			make new folder in folder myMovieFolder with properties {name:myFolder}
		end if
		--
	end tell
end finderstuff
--
on GetTimeFromDate(theDate)
	set theTimeRec to {hours:0, minutes:0, seconds:0, hours12:0, isAM:false}
	set numSecondsNow to time of theDate
	set hours of theTimeRec to numSecondsNow div 3600
	set minutes of theTimeRec to (numSecondsNow div 60) mod 60
	set seconds of theTimeRec to numSecondsNow mod 60
	set tmpHours to hours of theTimeRec
	if tmpHours = 0 then
		set isAM of theTimeRec to true
		set hours12 of theTimeRec to 12
	else if tmpHours > 12 then
		set isAM of theTimeRec to false
		set hours12 of theTimeRec to tmpHours - 12
	else
		set isAM of theTimeRec to true
		set hours12 of theTimeRec to tmpHours
	end if
	
	return (theTimeRec)
end GetTimeFromDate
--
on growlinfo(myTitle, myStartDate, myExportformat, Status)
	set myApp to ""
	
	tell application "System Events" to set GrowlRunning to ((application processes whose (name is equal to "GrowlHelperApp")) count)
	
	if GrowlRunning  1 then
		try
			set myApp to "GrowlHelperApp"
			-- Inits
			set appName to "\"EyeTV Exporter Notification Script\""
			set notificationName to "\"EyeTV Exporter Notification\""
			set notifs to "{\"EyeTV Exporter Notification\"}"
			-- end Inits
			-- Register GrowlNotification 
			set myrunscript to "register as application " & appName & " all notifications " & notifs & " default notifications " & notifs & " icon of application \"EyeTV\""
			tell application myApp to run script myrunscript
			-- end Register GrowlNotification
			-- Do Growlnotification
			if Status is "export" then
				set myDescription to "will be exported from startdate " & myStartDate & " as " & myExportformat & "."
				set myrunscript to "notify with name " & notificationName & " application name " & appName & " title " & "\"" & myTitle & "\"" & " description " & "\"" & myDescription & "\""
				tell application myApp to run script myrunscript
			else if Status is "skip" then
				set myDescription to "from startdate " & myStartDate & " as " & myExportformat & " will be skipped because the file already exists."
				set myrunscript to "notify with name " & notificationName & " application name " & appName & " title " & "\"" & myTitle & "\"" & " description " & "\"" & myDescription & "\""
				tell application myApp to run script myrunscript
			end if
			-- End do Growlnotification
		end try
	end if
end growlinfo
--
--
on checkfile(myPath)
	tell application "Finder"
		try
			alias myPath
			set myResult to true
		on error
			set myResult to false
		end try
	end tell
	return myResult
end checkfile
--
on getRecCount()
	tell application "EyeTV"
		try
			launch -- apps can't receive the run event except at startup, from the Finder, so this causes an error 
		on error
			display dialog "Could not start EyeTV, perhaps it is not installed" buttons {"OK"} default button 1
		end try
		set recCount to count of recordings
		return recCount
	end tell
end getRecCount
--
on eyetv_export(myExportformat, myTargetPath, initdelrec)
	tell application "EyeTV"
		try
			launch -- apps can't receive the run event except at startup, from the Finder, so this causes an error 
		on error
			display dialog "Could not start EyeTV, perhaps it is not installed" buttons {"OK"} default button 1
		end try
		--
		set myDelList to {}
		repeat with myRecording in recordings
			-- Inits
			set myTitle to my convertText(title of myRecording)
			set myDate to start time of myRecording
			set myDateString to date string of myDate & " " & my timeString(myDate)
			set myInfostring to myTitle & ", " & myDateString & ", " & myExportformat
			set myFilenamenoEx to myTargetPath & myInfostring
			
			-- Main Export
			if myExportformat is "QuickTime" then
				set myFilepath to myFilenamenoEx & ".mov"
				set filecheck to my checkfile(myFilepath)
				if filecheck is false then
					set Status to "export"
					export from myRecording to file myFilepath as QuickTime
				else
					set Status to "skip"
				end if
			else if myExportformat is "MPEGPS" then
				set myFilepath to myFilenamenoEx & ".mpg"
				set filecheck to my checkfile(myFilepath)
				if filecheck is false then
					set Status to "export"
					export from myRecording to file myFilepath as MPEGPS
				else
					set Status to "skip"
				end if
			else if myExportformat is "MPEGES" then
				set myFilepath to myFilenamenoEx & ".mpv"
				set filecheck to my checkfile(myFilepath)
				if filecheck is false then
					set Status to "export"
					export from myRecording to file myFilepath as MPEGES
				else
					set Status to "skip"
				end if
			else if myExportformat is "DV" then
				set myFilepath to myFilenamenoEx & ".dv"
				set filecheck to my checkfile(myFilepath)
				if filecheck is false then
					set Status to "export"
					export from myRecording to file myFilepath as DV
				else
					set Status to "skip"
				end if
			else if myExportformat is "DV169" then
				set myFilepath to myFilenamenoEx & ".dv"
				set filecheck to my checkfile(myFilepath)
				if filecheck is false then
					set Status to "export"
					export from myRecording to file myFilepath as DV169
				else
					set Status to "skip"
				end if
			else if myExportformat is "Movie" then
				set myFilepath to myFilenamenoEx & ".mov"
				set filecheck to my checkfile(myFilepath)
				if filecheck is false then
					set Status to "export"
					export from myRecording to file myFilepath as Movie
				else
					set Status to "skip"
				end if
			else if myExportformat is "AVI" then
				set myFilepath to myFilenamenoEx & ".avi"
				
				set filecheck to my checkfile(myFilepath)
				if filecheck is false then
					set Status to "export"
					export from myRecording to file myFilepath as AVI
				else
					set Status to "skip"
				end if
			else if myExportformat is "MPEG4" then
				set myFilepath to myFilenamenoEx & ".mp4"
				set filecheck to my checkfile(myFilepath)
				if filecheck is false then
					set Status to "export"
					export from myRecording to file myFilepath as MPEG4
				else
					set Status to "skip"
				end if
			else if myExportformat is "ThreeG" then
				set myFilepath to myFilenamenoEx & ".3gp"
				set filecheck to my checkfile(myFilepath)
				if filecheck is false then
					set Status to "export"
					export from myRecording to file myFilepath as ThreeG
				else
					set Status to "skip"
				end if
			else if myExportformat is "DivX" then
				set myFilepath to myFilenamenoEx & ".divx"
				set filecheck to my checkfile(myFilepath)
				if filecheck is false then
					set Status to "export"
					export from myRecording to file myFilepath as DivX
				else
					set Status to "skip"
				end if
			else if myExportformat is "HDV720p" then
				set myFilepath to myFilenamenoEx & ".mov"
				set filecheck to my checkfile(myFilepath)
				if filecheck is false then
					set Status to "export"
					export from myRecording to file myFilepath as HDV720p
				else
					set Status to "skip"
				end if
			else if myExportformat is "HDV1080i" then
				set myFilepath to myFilenamenoEx & ".mov"
				set filecheck to my checkfile(myFilepath)
				if filecheck is false then
					set Status to "export"
					export from myRecording to file myFilepath as QuickTime
				else
					set Status to "skip"
				end if
				export from myRecording to file myFilepath as HDV1080i
			else if myExportformat is "iTunes" then
				set myFilepath to myFilenamenoEx & ".m4a"
				set filecheck to my checkfile(myFilepath)
				if filecheck is false then
					set Status to "export"
					export from myRecording to file myFilepath as iTunes
				else
					set Status to "skip"
				end if
			else if myExportformat is "PSP" then
				set myFilepath to myFilenamenoEx & ".MP4"
				set filecheck to my checkfile(myFilepath)
				if filecheck is false then
					set Status to "export"
					export from myRecording to file myFilepath as PSP
				else
					set Status to "skip"
				end if
			else if myExportformat is "PSPH264" then
				set myFilepath to myFilenamenoEx & ".MP4"
				set filecheck to my checkfile(myFilepath)
				if filecheck is false then
					set Status to "export"
					export from myRecording to file myFilepath as PSPH264
				else
					set Status to "skip"
				end if
			else if myExportformat is "H264" then
				set myFilepath to myFilenamenoEx & ".mp4"
				set filecheck to my checkfile(myFilepath)
				if filecheck is false then
					set Status to "export"
					export from myRecording to file myFilepath as H264
				else
					set Status to "skip"
				end if
			else if myExportformat is "AAC" then
				set myFilepath to myFilenamenoEx & ".m4a"
				set filecheck to my checkfile(myFilepath)
				if filecheck is false then
					set Status to "export"
					export from myRecording to file myFilepath as AppleLossless
				else
					set Status to "skip"
				end if
			else if myExportformat is "iPod" then
				set myFilepath to myFilenamenoEx & ".m4v"
				set filecheck to my checkfile(myFilepath)
				if filecheck is false then
					set Status to "export"
					export from myRecording to file myFilepath as iPod
				else
					set Status to "skip"
				end if
			else if myExportformat is "iPodMP4" then
				set myFilepath to myFilenamenoEx & ".m4v"
				set filecheck to my checkfile(myFilepath)
				if filecheck is false then
					set Status to "export"
					export from myRecording to file myFilepath as iPodMP4
				else
					set Status to "skip"
				end if
			else if myExportformat is "AppleLossless" then
				set myFilepath to myFilenamenoEx & ".m4a"
				set filecheck to my checkfile(myFilepath)
				if filecheck is false then
					set Status to "export"
					export from myRecording to file myFilepath as AppleLossless
				else
					set Status to "skip"
				end if
			end if
			my growlinfo(myTitle, myDateString, myExportformat, Status)
			if (initdelrec is "Yes") then
				set myPrompttext to "Do you want to delete " & myTitle & " from " & myDateString & "?"
				if my booleanChoose(myPrompttext, "Yes", "No", "Delete Recording") is "Yes" then
					set myDelList to myDelList & {location of myRecording as string}
					--display dialog myDelList
				end if
			end if
			
			-- Check if EyeTV Exports more than one Movie
			repeat until my counteyetvproc() is less than 2
				delay (5)
			end repeat
			-- end check
		end repeat
	end tell
	if (initdelrec is "Yes") then
		my delChoosenRec(myDelList)
	end if
end eyetv_export
--
on delChoosenRec(myDelList)
	display dialog "hallo welt"
	log myDelList
	tell application "EyeTV"
		repeat with mydelRecording in myDelList
			repeat with myRecording in recordings
				log title of myRecording as string
				log mydelRecording
				delay 2
				
				if location of myRecording as string is mydelRecording then
					log title of myRecording
					log mydelRecording
					delay 2
					display dialog "Yes"
					delete myRecording
				end if
			end repeat
		end repeat
	end tell
end delChoosenRec
--
on checkBusy(myRecording)
	tell application "EyeTV"
		set myResult to (busy of myRecording)
		return myResult
	end tell
end checkBusy
--
on booleanChoose(myPromptext, answer1, answer2, myTitle)
	display dialog myPromptext buttons {answer1, answer2} default button 2 with title myTitle
	if button returned of result = answer1 then
		set myResult to answer1
	else if button returned of result = answer2 then
		set myResult to answer2
	end if
	return myResult
end booleanChoose
--
on startdialog()
	display dialog "Do you want to choose folder or do want to use the standard folder?" buttons {"Standard", "Choose folder", "Abort"} default button 1 with title "EyeTV Exporter"
	if button returned of result = "Standard" then
		set myResult to "Standard"
	else if button returned of result = "Choose folder" then
		set myResult to "Choose folder"
	else
		set myResult to "Abort"
	end if
	return myResult
end startdialog
--
on warningdialog(recCount)
	if recCount is greater than 1 then
		
		display dialog "You will export " & recCount & " recordings. Your system will perhaps be react very slow while exporting. Look for your free disk space. Do you want to proceed?" buttons {"Yes", "No"} default button 2 with icon stop with title "EyeTV Exporter"
		
		if button returned of result = "Yes" then
			set myResult to "Yes"
		else
			set myResult to "No"
		end if
	else
		set myResult to "Yes"
	end if
	return myResult
end warningdialog
--
on exportdialog()
	set theExportformats to {"QuickTime", "MPEGPS", "MPEGES", "DV", "DV169", "Movie", "AVI", "MPEG4", "ThreeG", "DivX", "HDV720p", "HDV1080i", "iTunes", "PSP", "H264", "PSPH264", "AAC", "AppleLossless", "iPod", "iPodMP4"}
	choose from list theExportformats with title "Export Format"
	set myExportformat to result as string
	return myExportformat
end exportdialog
--
on norecdialog()
	display dialog "You have no EyeTV recodings to export" buttons {"OK"} default button 1
end norecdialog
--
on setupfolder(myFolderchoice)
	if myFolderchoice is "Standard" then
		set myMovieFolder to ((path to movies folder from user domain) as text)
		set myTargetPath to myMovieFolder & myFoldername & ":"
	else if myFolderchoice is "Choose folder" then
		set myMovieFolder to (choose folder) as string
		set myTargetPath to myMovieFolder
	end if
	return {myTargetPath, myMovieFolder}
end setupfolder
--
-- Inits
--
-- set up export formats
--
set recCount to getRecCount()
--
if recCount is greater than 0 then
	if warningdialog(recCount) is "Yes" then
		set myFolderchoice to startdialog()
		--	set initdelrec to booleanChoose("Do you want to delete some of the recordings after export?", "Yes", "No", "Delete after export")
		set initdelrec to "No"
		if myFolderchoice is not "Abort" then
			set pathItems to setupfolder(myFolderchoice)
			finderstuff(item 1 of pathItems, item 2 of pathItems, myFoldername)
			set myExportformat to exportdialog()
			if myExportformat is not "false" then
				eyetv_export(myExportformat, item 1 of pathItems, initdelrec)
			end if
		end if
	end if
else
	norecdialog()
end if